% centx             : cell centroids (x coordinates)
% centy             : cell centroids (y coordinates)
%
% FOR ELIMINATING THE EDGES BETWEEN TWO OBJECTS THAT ARE CLOSE TO A
% BOUNDARY POINT
% dx                : x dimension of the image
% dy                : y dimension of the image
% wthr              : remove the edges if both of its end points are close
%                     to a boundary point. wthr determines such points.
%                     if wthr == -1, it ignores these conditions
function adjM = constructTriangulationGraph(centx, centy, dx, dy, wthr)

tri = delaunay(centx, centy);
% triplot(tri, centx, centy)

centNo = length(centx);
triNo = size(tri, 1);
adjM = zeros(centNo, centNo);
for i = 1 : triNo
    n1 = tri(i, 1);
    n2 = tri(i, 2);
    n3 = tri(i, 3);
    
    if wthr == -1
        adjM(n1, n2) = 1;
        adjM(n2, n1) = 1;

        adjM(n1, n3) = 1;
        adjM(n3, n1) = 1;

        adjM(n2, n3) = 1;
        adjM(n3, n2) = 1;
    else
        r1 = farFromBnd(centx(n1), centy(n1), dx, dy, wthr);
        r2 = farFromBnd(centx(n2), centy(n2), dx, dy, wthr);
        r3 = farFromBnd(centx(n3), centy(n3), dx, dy, wthr);
    
        if r1 || r2
            adjM(n1, n2) = 1;
            adjM(n2, n1) = 1;
        end

        if r2 || r3
            adjM(n2, n3) = 1;
            adjM(n3, n2) = 1;
        end

        if r1 || r3
            adjM(n3, n1) = 1;
            adjM(n1, n3) = 1;
        end
    end
end

end
%--------------------------------------------------------------------%
%-------------- Check if a point is close to a boundary -------------%
%--------------------------------------------------------------------%
function res = farFromBnd(centx, centy, dx, dy, wthr)

res = (centx >= wthr) && (centx < dx - wthr) && ...
      (centy >= wthr) && (centy < dy - wthr);

end
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%

