function allFeatures = extractGraphFeatures(adjM, centx, centy)

nodeNo = size(adjM, 1);

for i = 1 : nodeNo
    features = calculateDegreeAndClusteringCoefficient(adjM, centx, centy, i);
    %hops = bfs(adjM, i);
    %features.Eccentricity = max(hops);
    
    allFeatures(i, 1) = features;
end

end
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%
function features = calculateDegreeAndClusteringCoefficient(adjM, centx, centy, selectedNode)

nodeNo = size(adjM, 1);

degree = 0;
existingEdges = 0;
for i = 1 : nodeNo
    if adjM(selectedNode, i)
        degree = degree + 1;
        for j = i+1 : nodeNo
            if adjM(selectedNode, j)
                if adjM(i, j)
                    existingEdges = existingEdges + 1;
                end
            end
        end
    end
end

% To calculate the average, standard deviation, min-to-max ratio of the
% edge lengths from the node to its immediate neighbors
L = zeros(degree, 1);
curr = 1;
for i = 1 : nodeNo
    if adjM(selectedNode, i)
        L(curr) = sqrt((centx(selectedNode) - centx(i))*(centx(selectedNode) - centx(i)) + ...
                       (centy(selectedNode) - centy(i))*(centy(selectedNode) - centy(i)));
        curr = curr + 1;
    end
end

features.Degree = degree;
if degree
    features.ClusteringCoefficient = 2 * existingEdges / (degree * (degree - 1));
    features.AvgEdgeLength = mean(L);
    features.StdEdgeLength = std(L);
    features.Min2MaxEdgeLength = min(L) / max(L);
else
    features.ClusteringCoefficient = 0;
    features.AvgEdgeLength = 0;
    features.StdEdgeLength = 0;
    features.Min2MaxEdgeLength = 0;
end

end
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%
function hops = bfs(adjM, startId)

nodeNo = size(adjM, 1);
hops = zeros(nodeNo, 1) - 1;
queue = zeros(nodeNo, 1);

hops(startId) = 0;
queue(1) = startId;

qs = 1;
qe = 1;
while qs <= qe
    curr = queue(qs);
    qs = qs + 1;

    for ng = 1 : nodeNo
        if adjM(curr, ng)
            if hops(ng) == -1
                hops(ng) = hops(curr) + 1;
                qe = qe + 1;
                queue(qe) = ng;
            end
        end
    end
end
    
end
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%
%--------------------------------------------------------------------%


